<?php
/*
Plugin Name: 5sec Run PHP
Plugin URI: http://5sec-run-php.webfactoryltd.com/
Description: Safely run PHP code in posts, pages, widgets, excerpts and RSS. Usage: [php]your code[/php]
Author: Web factory Ltd
Version: 1.0
Author URI: http://www.soft2day.cu.cc/
*/

// **********************************
// !! DO NOT TOUCH BELOW THIS LINE !!
// **********************************

class wf_php {
  static public $include_css = false;
  
  public function init() {
    if (is_admin()) {
      // add PHP menu to admin-tools
      add_action('admin_menu', array('wf_php', 'admin_menu'));
      
      // settings registration
      add_action('admin_init', array('wf_php', 'register_settings'));
      
      // aditional link in plugin description
      add_action('plugin_action_links_' . basename(dirname(__FILE__)) . '/' . basename(__FILE__), array('wf_php', 'plugin_action_links'));
      
      // register server-side AJAX callback
      add_action('wp_ajax_wf_php_dismiss_message', array('wf_php', 'ajax_callback'));
      
      // if the plugin was installed/reinstalled set settings to default
      self::default_settings(false);
      
      // check if user has visual editor enabled
      add_action('admin_footer', array('wf_php', 'warning2'));
    } else {
      // add shortcode support in sidebar text widget
      if (has_filter('widget_text', 'do_shortcode') === false) {
        add_filter('widget_text', 'do_shortcode');
      }
      
      // add shortcode functionality for excerpt
      if (has_filter('the_excerpt', 'do_shortcode') === false) {
        add_filter('the_excerpt', 'do_shortcode');
      }
      
      // add shortcode functionality for RSS content
      if (has_filter('the_content_rss', 'do_shortcode') === false) {
        add_filter('the_content_rss', 'do_shortcode');
      }
      
      // add shortcode functionality for RSS excerpt
      if (has_filter('the_excerpt_rss', 'do_shortcode') === false) {
        add_filter('the_excerpt_rss', 'do_shortcode');
      }
      
      // use footer to inject CSS if need for <pre> code formatting
      add_action('wp_footer', array('wf_php', 'wp_footer'));
      
      // remove formatting on php shortcode content
      remove_filter('the_content', 'wpautop');
      remove_filter('the_content', 'wptexturize');
      add_filter('the_content', array('wf_php', 'content_filter'), 99, 1);
    }
    
    // add shortcode
    global $shortcode_tags;
    $shortcode = get_option('wf_php');
    $shortcode = $shortcode['shortcode'];
    if (isset($shortcode_tags[$shortcode])) {
      add_action('admin_footer', array('wf_php', 'warning'));
    } else {
      add_shortcode($shortcode, array('wf_php', 'shortcode'));
    }
  } // init
  
  // add settings link to plugins page
  function plugin_action_links($links) {
    $settings_link = '<a href="options-general.php?page=wf-php">Settings</a>';
    array_unshift($links, $settings_link);

    return $links;
  } // plugin_action_links
  
  function content_filter($content) {
    $out = '';
    $shortcode = get_option('wf_php');
    $shortcode = $shortcode['shortcode'] . '-raw';
    
    $pattern = '{(\[' . $shortcode. '\].*?\[/' . $shortcode . '\])}is';
    $pattern_pieces = '{\[' . $shortcode . '\](.*?)\[/' . $shortcode . '\]}is';
    $pieces = preg_split($pattern, $content, -1, PREG_SPLIT_DELIM_CAPTURE);

    foreach ($pieces as $piece) {
      if (preg_match($pattern_pieces, $piece, $matches)) {
        $out .= $matches[1];
      } else {
        $out .= wptexturize(wpautop($piece));
      }
    } // foreach
    
    return $out;
  } // content_filter
  
  // include CSS if needed
  function wp_footer() {
    if (!self::$include_css) {
      return;
    }

    $bg = plugins_url('/pre-code-bg.gif', __FILE__);
    
    echo '<style type="text/css">' . "\n";    
    echo 'pre.php-code {
      font-family: "Courier New", Courier, monospace !important;
      font-size: 12px !important;
      line-height: 20px !important;
      letter-spacing: 1px;
      padding: 0 18px;
      display: block;
      margin: 5px 0;
      background: #f0f0f0 url(' . $bg . ') repeat left top !important;
      overflow: auto;
      overflow-y: hidden;
      }' . "\n";
    echo '</style>' . "\n";
  } // wp_footer
  
  // create the admin menu item
  function admin_menu() {
    add_options_page('5sec Run PHP', '5sec Run PHP', 'manage_options', 'wf-php', array('wf_php', 'options_page'));
  } // admin_menu
  
  // all settinga are saved in one option
  function register_settings() {
    register_setting('wf_php', 'wf_php', array('wf_php', 'sanitize_settings'));
  } // register_settings
  
  // set default settings
  function default_settings($force = false) {
    $defaults['errors'] = 'hide';
    $defaults['action-content'] = 'run';
    $defaults['action-excerpt'] = 'run';
    $defaults['action-widget'] = 'run';
    $defaults['action-rss-content'] = 'run';
    $defaults['action-rss-excerpt'] = 'run';
    $defaults['shortcode'] = 'php';
    $defaults['dismiss_message'] = 0;
    
    $options = get_option('wf_php');
    
    if ($force || !$options || !$options['errors']) {
      update_option('wf_php', $defaults);
    }  
  } // default_settings
  
  // sanitize settings on save
  function sanitize_settings($values) {
    foreach ($values as $key => $value) {
      switch ($key) {
        case 'shortcode':
          $values[$key] = strtolower($value);
          $values[$key] = str_replace('[', '', $values[$key]);
          $values[$key] = str_replace(']', '', $values[$key]);
          $values[$key] = str_replace(' ', '', $values[$key]);
        break;
        default:
          $values[$key] = trim($value);
        break;
      }
    } // foreach
    
    return $values;
  } // sanitize_settings
  
  // detect where the shortcode was used
  function get_current_area() {
    $backtrace = debug_backtrace();
    foreach ($backtrace as $tmp) {
      if (in_array('the_content', $tmp['args'], 1)) {
        return 'content';
      }
      if (in_array('the_excerpt', $tmp['args'], 1)) {
        return 'excerpt';
      }
      if (in_array('widget_text', $tmp['args'], 1)) {
        return 'widget';
      }
      if (in_array('the_content_rss', $tmp['args'], 1)) {
        return 'rss-content';
      }
      if (in_array('the_excerpt_rss', $tmp['args'], 1)) {
        return 'rss-excerpt';
      }
    }
    
    return false;
  }
  
  // php shortcode logic
  public function shortcode($atts, $code = '') {
    global $post;
    $options = get_option('wf_php');
    $out = '';
    
    // where are we? content? widget? get default behaviour
    switch (self::get_current_area()) {
      case 'excerpt':
        $action = $options['action-excerpt'];
      break;
      case 'widget':
        $action = $options['action-widget'];
      break;
      case 'rss-content':
        $action = $options['action-rss-content'];
      break;
      case 'rss-excerpt':
        $action = $options['action-rss-excerpt'];
      break;
      case 'content':
      default;
        $action = $options['action-content'];
      break;
    }

    // parse attributes and set some default values
    $atts = shortcode_atts(array('errors'  => $options['errors'],
                                  'action' => $action),
                           $atts);
    
    // decide what action to perform
    switch ($atts['action']) {
      case 'delete':
        $out = '';
      break;
      case 'show':
        self::$include_css = true;
        $out = '<pre class="php-code">' . $code . "</pre>\n";
      break;
      case 'show-plain':
        $out = '<span class="php-code">' . $code . "</span>\n";
      break;
      case 'run':
      default:
        if ($atts['errors'] == 'show' || $atts['errors'] == 'hide') {
            $reporting = error_reporting();
            $display_errors = ini_get('display_errors');
        }
        if ($atts['errors'] == 'hide') {
          ini_set('display_errors', 0);
          error_reporting(0); 
        }
        if ($atts['errors'] == 'show') {
          ini_set('display_errors', 1);
          error_reporting(-1); 
        }
        // run the actual PHP code
        ob_start();
        eval($code);
        $out = ob_get_contents();
        ob_end_clean();
        if ($atts['errors'] == 'show' || $atts['errors'] == 'hide') {
          error_reporting($reporting);
          ini_set('display_errors', $display_errors);
        }
      break;
    }
    
    return $out;
  } // shortcode
  
  // helper function for creating select's options
  function create_select_options($options, $selected = null, $output = true) {
    $out = "\n";
    foreach ($options as $tmp) {
      if ($selected == $tmp['val']) {
        $out .= "<option selected=\"selected\" value=\"{$tmp['val']}\">{$tmp['label']}&nbsp;</option>\n";
      } else {
        $out .= "<option value=\"{$tmp['val']}\">{$tmp['label']}&nbsp;</option>\n";
      }
    }
    
    if($output) {
      echo $out;
    } else {
      return $out;
    }
  } // create_select_options
  
  // dismiss visual editor message
  function ajax_callback() {
    $options = get_option('wf_php');
    $options['dismiss_message'] = true;
    
    update_option('wf_php', $options);
  
    return true;
  } // ajax_callback
  
  // output the whole options page
  function options_page() {
    if (!current_user_can('manage_options'))  {
      wp_die('You do not have sufficient permissions to access this page.');
    }

    $options = get_option('wf_php');
    
    echo '<div class="wrap">
          <div class="icon32" id="icon-options-general"><br /></div>
          <h2>5sec Run PHP</h2>';

    echo '<form action="options.php" method="post">';
    settings_fields('wf_php');
    
    echo '<table class="form-table"><tbody>';

    $errors[] = array('val' => 'default', 'label' => 'Leave as it\'s configured by server and WordPress');
    $errors[] = array('val' => 'show', 'label' => 'Show all errors, warnings and notices');
    $errors[] = array('val' => 'hide', 'label' => 'Hide all errors, warnings and notices');
    echo '<tr valign="top">
    <th scope="row"><label for="errors">Error Reporting</label></th>
    <td><select id="errors" name="wf_php[errors]">';
      self::create_select_options($errors, $options['errors']);
    echo '</select> When developing it\'s useful to show all errors but in production environments please hide all errors. This option can be overwritten on a per-shortcode basis with "errors" parameter. Default: hide all errors, warnings and notices.</td>
    </tr>';
    
    $actions[] = array('val' => 'run', 'label' => 'Run PHP code');
    $actions[] = array('val' => 'delete', 'label' => 'Completely remove PHP code withouth running it');
    $actions[] = array('val' => 'show', 'label' => 'Show PHP code as formatted text in &lt;pre&gt; tags');
    $actions[] = array('val' => 'show-plain', 'label' => 'Show PHP code as unformatted text');
    echo '<tr valign="top">
    <th scope="row"><label for="action-content">Action for PHP Code in Content</label></th>
    <td><select id="action-content" name="wf_php[action-content]">';
      self::create_select_options($actions, $options['action-content']);
    echo '</select> This plugin is ment to run PHP code but you can use this aditional functionality to provide security or to show off your code. This option can be overwritten on a per-shortcode basis with "action" parameter. Default: run PHP code.</td>
    </tr>';
    
    echo '<tr valign="top">
    <th scope="row"><label for="action-excerpt">Action for PHP Code in Excerpt</label></th>
    <td><select id="action-excerpt" name="wf_php[action-excerpt]">';
      self::create_select_options($actions, $options['action-excerpt']);
    echo '</select> This plugin is ment to run PHP code but you can use this aditional functionality to provide security or to show off your code. This option can be overwritten on a per-shortcode basis with "action" parameter. Default: run PHP code.</td>
    </tr>';
    
    echo '<tr valign="top">
    <th scope="row"><label for="action-widget">Action for PHP Code in Text Widget</label></th>
    <td><select id="action-widget" name="wf_php[action-widget]">';
      self::create_select_options($actions, $options['action-widget']);
    echo '</select> This plugin is ment to run PHP code but you can use this aditional functionality to provide security or to show off your code. This option can be overwritten on a per-shortcode basis with "action" parameter. Default: run PHP code.</td>
    </tr>';
    
    echo '<tr valign="top">
    <th scope="row"><label for="action-rss-content">Action for PHP Code in RSS Content</label></th>
    <td><select id="action-rss-content" name="wf_php[action-rss-content]">';
      self::create_select_options($actions, $options['action-rss-content']);
    echo '</select> This plugin is ment to run PHP code but you can use this aditional functionality to provide security or to show off your code. This option can be overwritten on a per-shortcode basis with "action" parameter. Default: run PHP code.</td>
    </tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="action-rss-excerpt">Action for PHP Code in RSS Excerpt</label></th>
    <td><select id="action-rss-excerpt" name="wf_php[action-rss-excerpt]">';
      self::create_select_options($actions, $options['action-rss-excerpt']);
    echo '</select> This plugin is ment to run PHP code but you can use this aditional functionality to provide security or to show off your code. This option can be overwritten on a per-shortcode basis with "action" parameter. Default: run PHP code.</td>
    </tr>';
    
    echo '<tr valign="top">
    <th scope="row"><label for="shortcode">Shortcode Name</label></th>
    <td><input type="text" id="shortcode" style="width: 100px;" class="small-text" name="wf_php[shortcode]" value="' . $options['shortcode'] . '" />';
    echo ' Write only the shortcode name, without brackets. If the shortcode name you want is already taken you\'ll be notified after saving. Default: php.</td>
    </tr>';
    
    echo '</tbody></table>';
    echo '<p class="submit"><input type="submit" value="Save Changes" class="button-primary" name="Submit"></p>';
    echo '</form>';
    
    echo '<h2>Notes</h2>';
    
    echo '<ul style="list-style-type: circle; list-style-position: inside;">';
    echo '<li>You don\'t have to write opening and closing PHP tags (&lt;?php ?&gt;) in your code. Just use: <i>[php] // some code [/php]</i></li>';
    echo '<li>If you want to access post data for current post remember to put "global $post" in your code.</li>';
    echo '<li>If you want to interact with WordPress database object remember to put "global $wpdb" in your code.</li>';
    echo '<li>Error reporting can be defined on a per-shortcode basis using the "error" parameter. Ie: <i>[php errors="hide"] // some code [/php]</i>. Accepted values: default, show, hide.</li>';
    echo '<li>Action can be defined on a per-shortcode basis using the "action" parameter. Ie: <i>[php action="show"] // some code [/php]</i>. Accepted values: run, delete, show, show-plain.</li>';
    echo '<li>When action is set to "show-plain" the code is outputed in a &lt;span&gt; element with "php-code" class.</li>';
    echo '<li>When action is set to "show" the code is outputed in a &lt;pre&gt; element with "php-code" class and aditional CSS is included in your site\'s footer for code styling (please be sure your theme uses <i>wp_footer()</i> function call).</li>';
    echo '</ul>';
    echo '</div>';
  } // options_page

  // warning if [php] shortcode is already taken
  public function warning() {
    $shortcode = get_option('wf_php');
    $shortcode = $shortcode['shortcode'];
    echo '<div id="message-shortcode" class="error"><p><strong>5sec Run PHP is not active!</strong> The shortcode [' . $shortcode . '] is already in use by another plugin. Please <a href="options-general.php?page=wf-php">update the settings</a> and select a different shortcode.</p></div>';
  } // warning
  
  // check if user has visual editor enabled
  public function warning2() {
    global $current_user;
    $options = get_option('wf_php');
    
    if ($current_user->rich_editing == 'true' && !$options['dismiss_message']) {
      echo '<div id="message-visual-editor" class="error"><p>It\'s really <b>not advisable</b> to use the visual editor while writing PHP code in posts. Please disable it in <a href="profile.php">your profile</a>.<span style="float: right;"><a onclick="jQuery.post(ajaxurl, {\'action\':\'wf_php_dismiss_message\'}, function(response) { jQuery(\'#message-visual-editor\').hide(); }); return false;" href="#">Dismiss message</a></span></p></div>';
    }
  } // warning2
} // class wf_php

// hook our thing
add_action('init', array('wf_php', 'init'));
?>